@@ -4,11 +4,11 @@ pub mod resources;
 pub mod retry;
 
 use chrono::{DateTime, Utc};
-use lazy_static::lazy_static;
 use log::debug;
+use once_cell::sync::Lazy;
 use reqwest::{
     blocking::{multipart::Form, Client as HttpClient, Response as HttpResponse},
-    header::{HeaderMap, HeaderName, HeaderValue},
+    header::{self, HeaderMap, HeaderValue},
     IntoUrl, Proxy, Result as ReqwestResult,
 };
 use serde::{Deserialize, Serialize};
@@ -32,6 +32,10 @@ use crate::resources::{
         UpdateRequest as UpdateDatasetRequest, UpdateResponse as UpdateDatasetResponse,
     },
     email::{PutEmailsRequest, PutEmailsResponse},
+    project::{
+        CreateProjectRequest, CreateProjectResponse, GetProjectResponse, GetProjectsResponse,
+        UpdateProjectRequest, UpdateProjectResponse,
+    },
     source::{
         CreateRequest as CreateSourceRequest, CreateResponse as CreateSourceResponse,
         GetAvailableResponse as GetAvailableSourcesResponse, GetResponse as GetSourceResponse,
@@ -47,7 +51,7 @@ use crate::resources::{
         GetAvailableResponse as GetAvailableUsersResponse,
         GetCurrentResponse as GetCurrentUserResponse,
     },
-    ApiError, EmptySuccess, Response, SimpleApiError,
+    EmptySuccess, Response,
 };
 
 use crate::retry::{Retrier, RetryConfig};
@@ -75,6 +79,10 @@ pub use crate::{
             LabelDef, LabelDefPretrained, Name as LabelName, NewLabelDef, NewLabelDefPretrained,
             PretrainedId as LabelDefPretrainedId,
         },
+        label_group::{
+            LabelGroup, Name as LabelGroupName, NewLabelGroup, DEFAULT_LABEL_GROUP_NAME,
+        },
+        project::{NewProject, Project, ProjectName, UpdateProject},
         source::{
             FullName as SourceFullName, Id as SourceId, Identifier as SourceIdentifier,
             Name as SourceName, NewSource, Source, UpdateSource,
@@ -165,7 +173,7 @@ impl Client {
     /// List all visible sources.
     pub fn get_sources(&self) -> Result<Vec<Source>> {
         Ok(self
-            .get::<_, GetAvailableSourcesResponse, SimpleApiError>(self.endpoints.sources.clone())?
+            .get::<_, GetAvailableSourcesResponse>(self.endpoints.sources.clone())?
             .sources)
     }
 
@@ -173,16 +181,12 @@ impl Client {
     pub fn get_source(&self, source: impl Into<SourceIdentifier>) -> Result<Source> {
         Ok(match source.into() {
             SourceIdentifier::Id(source_id) => {
-                self.get::<_, GetSourceResponse, SimpleApiError>(
-                    self.endpoints.source_by_id(&source_id)?,
-                )?
-                .source
+                self.get::<_, GetSourceResponse>(self.endpoints.source_by_id(&source_id)?)?
+                    .source
             }
             SourceIdentifier::FullName(source_name) => {
-                self.get::<_, GetSourceResponse, SimpleApiError>(
-                    self.endpoints.source_by_name(&source_name)?,
-                )?
-                .source
+                self.get::<_, GetSourceResponse>(self.endpoints.source_by_name(&source_name)?)?
+                    .source
             }
         })
     }
@@ -191,11 +195,11 @@ impl Client {
     pub fn create_source(
         &self,
         source_name: &SourceFullName,
-        options: NewSource,
+        options: NewSource<'_>,
     ) -> Result<Source> {
         Ok(self
-            .put::<_, _, CreateSourceResponse, SimpleApiError>(
-                self.endpoints.source_by_name(&source_name)?,
+            .put::<_, _, CreateSourceResponse>(
+                self.endpoints.source_by_name(source_name)?,
                 CreateSourceRequest { source: options },
             )?
             .source)
@@ -205,11 +209,11 @@ impl Client {
     pub fn update_source(
         &self,
         source_name: &SourceFullName,
-        options: UpdateSource,
+        options: UpdateSource<'_>,
     ) -> Result<Source> {
         Ok(self
-            .post::<_, _, UpdateSourceResponse, SimpleApiError>(
-                self.endpoints.source_by_name(&source_name)?,
+            .post::<_, _, UpdateSourceResponse>(
+                self.endpoints.source_by_name(source_name)?,
                 UpdateSourceRequest { source: options },
                 Retry::Yes,
             )?
@@ -222,7 +226,7 @@ impl Client {
             SourceIdentifier::Id(source_id) => source_id,
             source @ SourceIdentifier::FullName(_) => self.get_source(source)?.id,
         };
-        self.delete::<_, SimpleApiError>(self.endpoints.source_by_id(&source_id)?)
+        self.delete::<_>(self.endpoints.source_by_id(&source_id)?)
     }
 
     /// Delete comments by id in a source.
@@ -235,7 +239,7 @@ impl Client {
             source @ SourceIdentifier::Id(_) => self.get_source(source)?.full_name(),
             SourceIdentifier::FullName(source_full_name) => source_full_name,
         };
-        self.delete_query::<_, _, SimpleApiError>(
+        self.delete_query::<_, _>(
             self.endpoints.comments_v1(&source_full_name)?,
             Some(&id_list_query(comments.iter().map(|uid| &uid.0))),
         )
@@ -268,10 +272,7 @@ impl Client {
             after,
             limit,
         };
-        self.get_query::<_, _, _, SimpleApiError>(
-            self.endpoints.comments(source_name)?,
-            Some(&query_params),
-        )
+        self.get_query::<_, _, _>(self.endpoints.comments(source_name)?, Some(&query_params))
     }
 
     /// Iterate through all comments in a source.
@@ -291,9 +292,7 @@ impl Client {
         comment_id: &'a CommentId,
     ) -> Result<Comment> {
         Ok(self
-            .get::<_, GetCommentResponse, SimpleApiError>(
-                self.endpoints.comment_by_id(source_name, comment_id)?,
-            )?
+            .get::<_, GetCommentResponse>(self.endpoints.comment_by_id(source_name, comment_id)?)?
             .comment)
     }
 
@@ -302,7 +301,7 @@ impl Client {
         source_name: &SourceFullName,
         comments: &[NewComment],
     ) -> Result<PutCommentsResponse> {
-        self.put::<_, _, _, SimpleApiError>(
+        self.put::<_, _, _>(
             self.endpoints.comments(source_name)?,
             PutCommentsRequest { comments },
         )
@@ -313,7 +312,7 @@ impl Client {
         source_name: &SourceFullName,
         comments: &[NewComment],
     ) -> Result<SyncCommentsResponse> {
-        self.post::<_, _, _, SimpleApiError>(
+        self.post::<_, _, _>(
             self.endpoints.sync_comments(source_name)?,
             SyncCommentsRequest { comments },
             Retry::Yes,
@@ -325,7 +324,7 @@ impl Client {
         bucket_name: &BucketFullName,
         emails: &[NewEmail],
     ) -> Result<PutEmailsResponse> {
-        self.put::<_, _, _, SimpleApiError>(
+        self.put::<_, _, _>(
             self.endpoints.put_emails(bucket_name)?,
             PutEmailsRequest { emails },
         )
@@ -355,7 +354,7 @@ impl Client {
             })?;
         let status = http_response.status();
         http_response
-            .json::<Response<EmptySuccess, SimpleApiError>>()
+            .json::<Response<EmptySuccess>>()
             .map_err(Error::BadJsonResponse)?
             .into_result(status)?;
         Ok(())
@@ -363,9 +362,7 @@ impl Client {
 
     pub fn get_datasets(&self) -> Result<Vec<Dataset>> {
         Ok(self
-            .get::<_, GetAvailableDatasetsResponse, SimpleApiError>(
-                self.endpoints.datasets.clone(),
-            )?
+            .get::<_, GetAvailableDatasetsResponse>(self.endpoints.datasets.clone())?
             .datasets)
     }
 
@@ -375,16 +372,12 @@ impl Client {
     {
         Ok(match dataset.into() {
             DatasetIdentifier::Id(dataset_id) => {
-                self.get::<_, GetDatasetResponse, SimpleApiError>(
-                    self.endpoints.dataset_by_id(&dataset_id)?,
-                )?
-                .dataset
+                self.get::<_, GetDatasetResponse>(self.endpoints.dataset_by_id(&dataset_id)?)?
+                    .dataset
             }
             DatasetIdentifier::FullName(dataset_name) => {
-                self.get::<_, GetDatasetResponse, SimpleApiError>(
-                    self.endpoints.dataset_by_name(&dataset_name)?,
-                )?
-                .dataset
+                self.get::<_, GetDatasetResponse>(self.endpoints.dataset_by_name(&dataset_name)?)?
+                    .dataset
             }
         })
     }
@@ -393,10 +386,10 @@ impl Client {
     pub fn create_dataset(
         &self,
         dataset_name: &DatasetFullName,
-        options: NewDataset,
+        options: NewDataset<'_>,
     ) -> Result<Dataset> {
         Ok(self
-            .put::<_, _, CreateDatasetResponse, SimpleApiError>(
+            .put::<_, _, CreateDatasetResponse>(
                 self.endpoints.dataset_by_name(dataset_name)?,
                 CreateDatasetRequest { dataset: options },
             )?
@@ -407,10 +400,10 @@ impl Client {
     pub fn update_dataset(
         &self,
         dataset_name: &DatasetFullName,
-        options: UpdateDataset,
+        options: UpdateDataset<'_>,
     ) -> Result<Dataset> {
         Ok(self
-            .post::<_, _, UpdateDatasetResponse, SimpleApiError>(
+            .post::<_, _, UpdateDatasetResponse>(
                 self.endpoints.dataset_by_name(dataset_name)?,
                 UpdateDatasetRequest { dataset: options },
                 Retry::Yes,
@@ -426,7 +419,7 @@ impl Client {
             DatasetIdentifier::Id(dataset_id) => dataset_id,
             dataset @ DatasetIdentifier::FullName(_) => self.get_dataset(dataset)?.id,
         };
-        self.delete::<_, SimpleApiError>(self.endpoints.dataset_by_id(&dataset_id)?)
+        self.delete::<_>(self.endpoints.dataset_by_id(&dataset_id)?)
     }
 
     /// Get labellings for a given a dataset and a list of comment UIDs.
@@ -436,7 +429,7 @@ impl Client {
         comment_uids: impl Iterator<Item = &'a CommentUid>,
     ) -> Result<Vec<AnnotatedComment>> {
         Ok(self
-            .get_query::<_, _, GetAnnotationsResponse, SimpleApiError>(
+            .get_query::<_, _, GetAnnotationsResponse>(
                 self.endpoints.get_labellings(dataset_name)?,
                 Some(&id_list_query(comment_uids.into_iter().map(|id| &id.0))),
             )?
@@ -458,9 +451,9 @@ impl Client {
     pub fn get_labellings_in_bulk(
         &self,
         dataset_name: &DatasetFullName,
-        query_parameters: GetLabellingsInBulk,
+        query_parameters: GetLabellingsInBulk<'_>,
     ) -> Result<GetAnnotationsResponse> {
-        self.get_query::<_, _, GetAnnotationsResponse, SimpleApiError>(
+        self.get_query::<_, _, GetAnnotationsResponse>(
             self.endpoints.get_labellings(dataset_name)?,
             Some(&query_parameters),
         )
@@ -471,10 +464,10 @@ impl Client {
         &self,
         dataset_name: &DatasetFullName,
         comment_uid: &CommentUid,
-        labelling: Option<&NewLabelling>,
+        labelling: Option<&[NewLabelling]>,
         entities: Option<&NewEntities>,
     ) -> Result<AnnotatedComment> {
-        self.post::<_, _, AnnotatedComment, SimpleApiError>(
+        self.post::<_, _, AnnotatedComment>(
             self.endpoints.post_labelling(dataset_name, comment_uid)?,
             UpdateAnnotationsRequest {
                 labelling,
@@ -486,7 +479,7 @@ impl Client {
 
     pub fn get_triggers(&self, dataset_name: &DatasetFullName) -> Result<Vec<Trigger>> {
         Ok(self
-            .get::<_, GetTriggersResponse, SimpleApiError>(self.endpoints.triggers(dataset_name)?)?
+            .get::<_, GetTriggersResponse>(self.endpoints.triggers(dataset_name)?)?
             .triggers)
     }
 
@@ -497,7 +490,7 @@ impl Client {
         limit: usize,
         continuation: Option<&Continuation>,
     ) -> Result<RecentCommentsPage> {
-        self.post::<_, _, RecentCommentsPage, SimpleApiError>(
+        self.post::<_, _, RecentCommentsPage>(
             self.endpoints.recent_comments(dataset_name)?,
             GetRecentRequest {
                 limit,
@@ -510,19 +503,19 @@ impl Client {
 
     pub fn get_current_user(&self) -> Result<User> {
         Ok(self
-            .get::<_, GetCurrentUserResponse, SimpleApiError>(self.endpoints.current_user.clone())?
+            .get::<_, GetCurrentUserResponse>(self.endpoints.current_user.clone())?
             .user)
     }
 
     pub fn get_users(&self) -> Result<Vec<User>> {
         Ok(self
-            .get::<_, GetAvailableUsersResponse, SimpleApiError>(self.endpoints.users.clone())?
+            .get::<_, GetAvailableUsersResponse>(self.endpoints.users.clone())?
             .users)
     }
 
-    pub fn create_user(&self, user: NewUser) -> Result<User> {
+    pub fn create_user(&self, user: NewUser<'_>) -> Result<User> {
         Ok(self
-            .put::<_, _, CreateUserResponse, SimpleApiError>(
+            .put::<_, _, CreateUserResponse>(
                 self.endpoints.users.clone(),
                 CreateUserRequest { user },
             )?
@@ -531,7 +524,7 @@ impl Client {
 
     pub fn get_statistics(&self, dataset_name: &DatasetFullName) -> Result<Statistics> {
         Ok(self
-            .post::<_, _, GetStatisticsResponse, SimpleApiError>(
+            .post::<_, _, GetStatisticsResponse>(
                 self.endpoints.statistics(dataset_name)?,
                 json!({}),
                 Retry::No,
@@ -543,11 +536,11 @@ impl Client {
     pub fn create_bucket(
         &self,
         bucket_name: &BucketFullName,
-        options: NewBucket,
+        options: NewBucket<'_>,
     ) -> Result<Bucket> {
         Ok(self
-            .put::<_, _, CreateBucketResponse, SimpleApiError>(
-                self.endpoints.bucket_by_name(&bucket_name)?,
+            .put::<_, _, CreateBucketResponse>(
+                self.endpoints.bucket_by_name(bucket_name)?,
                 CreateBucketRequest { bucket: options },
             )?
             .bucket)
@@ -555,7 +548,7 @@ impl Client {
 
     pub fn get_buckets(&self) -> Result<Vec<Bucket>> {
         Ok(self
-            .get::<_, GetAvailableBucketsResponse, SimpleApiError>(self.endpoints.buckets.clone())?
+            .get::<_, GetAvailableBucketsResponse>(self.endpoints.buckets.clone())?
             .buckets)
     }
 
@@ -565,16 +558,12 @@ impl Client {
     {
         Ok(match bucket.into() {
             BucketIdentifier::Id(bucket_id) => {
-                self.get::<_, GetBucketResponse, SimpleApiError>(
-                    self.endpoints.bucket_by_id(&bucket_id)?,
-                )?
-                .bucket
+                self.get::<_, GetBucketResponse>(self.endpoints.bucket_by_id(&bucket_id)?)?
+                    .bucket
             }
             BucketIdentifier::FullName(bucket_name) => {
-                self.get::<_, GetBucketResponse, SimpleApiError>(
-                    self.endpoints.bucket_by_name(&bucket_name)?,
-                )?
-                .bucket
+                self.get::<_, GetBucketResponse>(self.endpoints.bucket_by_name(&bucket_name)?)?
+                    .bucket
             }
         })
     }
@@ -587,7 +576,7 @@ impl Client {
             BucketIdentifier::Id(bucket_id) => bucket_id,
             bucket @ BucketIdentifier::FullName(_) => self.get_bucket(bucket)?.id,
         };
-        self.delete::<_, SimpleApiError>(self.endpoints.bucket_by_id(&bucket_id)?)
+        self.delete::<_>(self.endpoints.bucket_by_id(&bucket_id)?)
     }
 
     pub fn fetch_trigger_comments(
@@ -595,7 +584,7 @@ impl Client {
         trigger_name: &TriggerFullName,
         size: u32,
     ) -> Result<TriggerBatch> {
-        self.post::<_, _, _, SimpleApiError>(
+        self.post::<_, _, _>(
             self.endpoints.trigger_fetch(trigger_name)?,
             TriggerFetchRequest { size },
             Retry::No,
@@ -607,7 +596,7 @@ impl Client {
         trigger_name: &TriggerFullName,
         sequence_id: TriggerSequenceId,
     ) -> Result<()> {
-        self.post::<_, _, serde::de::IgnoredAny, SimpleApiError>(
+        self.post::<_, _, serde::de::IgnoredAny>(
             self.endpoints.trigger_advance(trigger_name)?,
             TriggerAdvanceRequest { sequence_id },
             Retry::No,
@@ -620,7 +609,7 @@ impl Client {
         trigger_name: &TriggerFullName,
         to_comment_created_at: DateTime<Utc>,
     ) -> Result<()> {
-        self.post::<_, _, serde::de::IgnoredAny, SimpleApiError>(
+        self.post::<_, _, serde::de::IgnoredAny>(
             self.endpoints.trigger_reset(trigger_name)?,
             TriggerResetRequest {
                 to_comment_created_at,
@@ -630,16 +619,67 @@ impl Client {
         Ok(())
     }
 
-    fn get<LocationT, SuccessT, ErrorT>(&self, url: LocationT) -> Result<SuccessT>
+    /// Gets a project.
+    pub fn get_project(&self, project_name: &ProjectName) -> Result<Project> {
+        let response =
+            self.get::<_, GetProjectResponse>(self.endpoints.project_by_name(project_name)?)?;
+        Ok(response.project)
+    }
+
+    /// Gets all projects.
+    pub fn get_projects(&self) -> Result<Vec<Project>> {
+        let response = self.get::<_, GetProjectsResponse>(self.endpoints.projects.clone())?;
+        Ok(response.projects)
+    }
+
+    /// Creates a new project.
+    pub fn create_project(
+        &self,
+        project_name: &ProjectName,
+        options: NewProject,
+        user_ids: &[UserId],
+    ) -> Result<Project> {
+        Ok(self
+            .put::<_, _, CreateProjectResponse>(
+                self.endpoints.project_by_name(project_name)?,
+                CreateProjectRequest {
+                    project: options,
+                    user_ids,
+                },
+            )?
+            .project)
+    }
+
+    /// Updates an existing project.
+    pub fn update_project(
+        &self,
+        project_name: &ProjectName,
+        options: UpdateProject,
+    ) -> Result<Project> {
+        Ok(self
+            .post::<_, _, UpdateProjectResponse>(
+                self.endpoints.project_by_name(project_name)?,
+                UpdateProjectRequest { project: options },
+                Retry::Yes,
+            )?
+            .project)
+    }
+
+    /// Deletes an existing project.
+    pub fn delete_project(&self, project_name: &ProjectName) -> Result<()> {
+        self.delete(self.endpoints.project_by_name(project_name)?)?;
+        Ok(())
+    }
+
+    fn get<LocationT, SuccessT>(&self, url: LocationT) -> Result<SuccessT>
     where
         LocationT: IntoUrl + Display + Clone,
         for<'de> SuccessT: Deserialize<'de>,
-        for<'de> ErrorT: Deserialize<'de> + ApiError,
     {
-        self.get_query::<LocationT, (), SuccessT, ErrorT>(url, None)
+        self.get_query::<LocationT, (), SuccessT>(url, None)
     }
 
-    fn get_query<LocationT, QueryT, SuccessT, ErrorT>(
+    fn get_query<LocationT, QueryT, SuccessT>(
         &self,
         url: LocationT,
         query: Option<&QueryT>,
@@ -648,7 +688,6 @@ impl Client {
         LocationT: IntoUrl + Display + Clone,
         QueryT: Serialize,
         for<'de> SuccessT: Deserialize<'de>,
-        for<'de> ErrorT: Deserialize<'de> + ApiError,
     {
         debug!("Attempting GET `{}`", url);
         let http_response = self
@@ -668,28 +707,22 @@ impl Client {
             })?;
         let status = http_response.status();
         http_response
-            .json::<Response<SuccessT, ErrorT>>()
+            .json::<Response<SuccessT>>()
             .map_err(Error::BadJsonResponse)?
             .into_result(status)
     }
 
-    fn delete<LocationT, ErrorT>(&self, url: LocationT) -> Result<()>
+    fn delete<LocationT>(&self, url: LocationT) -> Result<()>
     where
         LocationT: IntoUrl + Display + Clone,
-        for<'de> ErrorT: Deserialize<'de> + ApiError,
     {
-        self.delete_query::<LocationT, (), ErrorT>(url, None)
+        self.delete_query::<LocationT, ()>(url, None)
     }
 
-    fn delete_query<LocationT, QueryT, ErrorT>(
-        &self,
-        url: LocationT,
-        query: Option<&QueryT>,
-    ) -> Result<()>
+    fn delete_query<LocationT, QueryT>(&self, url: LocationT, query: Option<&QueryT>) -> Result<()>
     where
         LocationT: IntoUrl + Display + Clone,
         QueryT: Serialize,
-        for<'de> ErrorT: Deserialize<'de> + ApiError,
     {
         debug!("Attempting DELETE `{}`", url);
 
@@ -713,7 +746,7 @@ impl Client {
             })?;
         let status = http_response.status();
         http_response
-            .json::<Response<EmptySuccess, ErrorT>>()
+            .json::<Response<EmptySuccess>>()
             .map_err(Error::BadJsonResponse)?
             .into_result(status)
             .map_or_else(
@@ -730,7 +763,7 @@ impl Client {
             )
     }
 
-    fn post<LocationT, RequestT, SuccessT, ErrorT>(
+    fn post<LocationT, RequestT, SuccessT>(
         &self,
         url: LocationT,
         request: RequestT,
@@ -740,7 +773,6 @@ impl Client {
         LocationT: IntoUrl + Display + Clone,
         RequestT: Serialize,
         for<'de> SuccessT: Deserialize<'de>,
-        for<'de> ErrorT: Deserialize<'de> + ApiError,
     {
         debug!("Attempting POST `{}`", url);
         let do_request = || {
@@ -761,12 +793,12 @@ impl Client {
         })?;
         let status = http_response.status();
         http_response
-            .json::<Response<SuccessT, ErrorT>>()
+            .json::<Response<SuccessT>>()
             .map_err(Error::BadJsonResponse)?
             .into_result(status)
     }
 
-    fn put<LocationT, RequestT, SuccessT, ErrorT>(
+    fn put<LocationT, RequestT, SuccessT>(
         &self,
         url: LocationT,
         request: RequestT,
@@ -775,7 +807,6 @@ impl Client {
         LocationT: IntoUrl + Display + Clone,
         RequestT: Serialize,
         for<'de> SuccessT: Deserialize<'de>,
-        for<'de> ErrorT: Deserialize<'de> + ApiError,
     {
         debug!("Attempting PUT `{}`", url);
         let http_response = self
@@ -792,7 +823,7 @@ impl Client {
             })?;
         let status = http_response.status();
         http_response
-            .json::<Response<SuccessT, ErrorT>>()
+            .json::<Response<SuccessT>>()
             .map_err(Error::BadJsonResponse)?
             .into_result(status)
     }
@@ -944,6 +975,7 @@ struct Endpoints {
     buckets: Url,
     users: Url,
     current_user: Url,
+    projects: Url,
 }
 
 impl Endpoints {
@@ -978,6 +1010,12 @@ impl Endpoints {
                 source,
                 message: "Could not build URL for users resources.".to_owned(),
             })?;
+        let projects =
+            base.join("/api/_private/projects")
+                .map_err(|source| Error::UrlParseError {
+                    source,
+                    message: "Could not build URL for project resources.".to_owned(),
+                })?;
         Ok(Endpoints {
             base,
             datasets,
@@ -985,6 +1023,7 @@ impl Endpoints {
             buckets,
             users,
             current_user,
+            projects,
         })
     }
 
@@ -1234,6 +1273,18 @@ impl Endpoints {
                 ),
             })
     }
+
+    fn project_by_name(&self, project_name: &ProjectName) -> Result<Url> {
+        self.base
+            .join(&format!("/api/_private/projects/{}", project_name.0))
+            .map_err(|source| Error::UrlParseError {
+                source,
+                message: format!(
+                    "Could not build URL for project resource with name `{}`.",
+                    project_name.0
+                ),
+            })
+    }
 }
 
 fn build_http_client(config: &Config) -> Result<HttpClient> {
@@ -1249,7 +1300,7 @@ fn build_http_client(config: &Config) -> Result<HttpClient> {
 fn build_headers(config: &Config) -> Result<HeaderMap> {
     let mut headers = HeaderMap::new();
     headers.insert(
-        AUTH_HEADER_NAME.clone(),
+        header::AUTHORIZATION,
         HeaderValue::from_str(&format!("Bearer {}", &config.token.0)).map_err(|_| {
             Error::BadToken {
                 token: config.token.0.clone(),
@@ -1266,11 +1317,8 @@ fn id_list_query<'a>(ids: impl Iterator<Item = &'a String>) -> Vec<(&'static str
     ids.map(|id| ("id", id.as_str())).collect()
 }
 
-lazy_static! {
-    static ref AUTH_HEADER_NAME: HeaderName = HeaderName::from_static("authorization");
-    pub static ref DEFAULT_ENDPOINT: Url =
-        Url::parse("https://reinfer.io").expect("Default URL is well-formed");
-}
+pub static DEFAULT_ENDPOINT: Lazy<Url> =
+    Lazy::new(|| Url::parse("https://reinfer.io").expect("Default URL is well-formed"));
 
 #[cfg(test)]
 mod tests {
