@@ -1,15 +1,17 @@
 use chrono::{DateTime, Utc};
+use ordered_float::NotNan;
 use serde::{Deserialize, Serialize};
 use std::str::FromStr;
 
 use crate::error::{Error, Result};
 
 use super::{
-    comment::{Comment, CommentFilter, Entity, LabelName, PredictedLabel},
+    comment::{Comment, CommentFilter, Entity, PredictedLabel},
     dataset::{FullName as DatasetFullName, Id as DatasetId},
+    label_def::Name as LabelName,
 };
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct Name(pub String);
 
 #[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
@@ -18,7 +20,7 @@ pub struct Id(pub String);
 #[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct SequenceId(pub String);
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct FullName {
     pub dataset: DatasetFullName,
     pub trigger: Name,
@@ -62,7 +64,7 @@ pub struct Trigger {
 pub struct LabelFilter {
     pub label: LabelName,
     pub model_version: UserModelVersion,
-    pub threshold: f64,
+    pub threshold: NotNan<f64>,
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize)]
